import json
import os
import platform
import warnings
from enum import Enum
from typing import Dict, Optional, TypedDict, Union

import jupyterlab
import jupyterlab_server

from aext_shared.__about__ import __version__ as extension_version


class ConfigValueFactory:
    """
    A factory for obtaining config values.

    Values can be set by environment variable with a default fallback.
    """

    def __init__(self) -> None:
        try:
            # cloud specific configuration defaults
            # This import will succeed in the cloud package because the associated file in included.
            # The file is not included in the local package and will cause an ImportError.
            # This is handled below to load local specific defaults.
            from .config_defaults_cloud import config_defaults_cloud

            if os.environ.get("DEBUG_AEXT_SHARED_LOCAL", False):
                raise ImportError
            self.defaults = config_defaults_cloud
        except ImportError:
            from .config_defaults_local import config_defaults_local

            self.defaults = config_defaults_local

    def __getitem__(self, key) -> str:
        return os.environ.get(key, self.defaults.get(key, None))


class OSPlatform(TypedDict):
    name: str
    system: str
    version: str
    release: str


class JupyterLab(TypedDict):
    server: str
    client: str


class Extension(TypedDict):
    version: str


class Environment(str, Enum):
    # Ignore the type errors for now. In the future, once the minimum python is >= 3.11,
    # switch this to StrEnum: https://stackoverflow.com/questions/58608361/string-based-enum-in-python
    # NOTE: these 3 envs will be deprecated
    development: str = "development"  # type: ignore
    production: str = "production"  # type: ignore
    local: str = "local"  # type: ignore

    # new definition for environments
    cloud_production: str = "cloud_production"  # type: ignore
    local_production: str = "local_production"  # type: ignore
    cloud_development: str = "cloud_development"  # type: ignore
    local_development: str = "local_development"  # type: ignore
    github_actions: str = "github_actions"  # type: ignore

    @classmethod
    def is_local(cls, env: "Environment") -> bool:
        return env in [cls.local_development, cls.local_production, cls.local]

    @classmethod
    def is_cloud(cls, env: "Environment") -> bool:
        return env in [cls.cloud_production, cls.cloud_development]

    @classmethod
    def is_development(cls, env: "Environment") -> bool:
        return env in [cls.cloud_development, cls.local_development]

    @classmethod
    def is_production(cls, env: "Environment") -> bool:
        return env in [cls.cloud_production, cls.local_production]


class API(TypedDict):
    domain: str
    url: str


class Org(TypedDict):
    url: str
    api: str


class StaticContent(TypedDict):
    url: str


class AnacondaAuth(TypedDict):
    domain: str
    token_endpoint: str


class IAM(TypedDict):
    url: str


class Notebooks(TypedDict):
    url: str


class Learning(TypedDict):
    url: str


class AssistantEnvironmentType(str, Enum):
    # Ignore the type errors for now. In the future, once the minimum python is >= 3.11,
    # switch this to StrEnum: https://stackoverflow.com/questions/58608361/string-based-enum-in-python
    # Possible values:
    # - local-dev for local development
    # - cloud-notebooks-prod for users of Anaconda Cloud
    # - cloud-notebooks-staging for users of Anaconda Cloud staging
    # - local-notebooks-dev for developing the Assistant for users of Jupyter Lab working locally
    # - local-notebooks-prod for users of Jupyter Lab working locally
    # - enterprise-notebooks-dev for developing the Assistant for users of AE5
    # - enterprise-notebooks-prod for users of AE5
    localDev: str = "local-dev"  # type: ignore
    cloudNotebooksProd: str = "cloud-notebooks-prod"  # type: ignore
    cloudNotebooksStaging: str = "cloud-notebooks-staging"  # type: ignore
    localNotebooksDev: str = "local-notebooks-dev"  # type: ignore
    localNotebooksProd: str = "local-notebooks-prod"  # type: ignore
    enterpriseNotebooksDev: str = "enterprise-notebooks-dev"  # type: ignore
    enterpriseNotebooksProd: str = "enterprise-notebooks-prod"  # type: ignore


class AssistantEnvironmentAPIUrl(str, Enum):
    # Ignore the type errors for now. In the future, once the minimum python is >= 3.11,
    # switch this to StrEnum: https://stackoverflow.com/questions/58608361/string-based-enum-in-python
    # Possible values:
    # - https://assistant.anaconda.cloud (after migration: https://anaconda.com/api/assistant)
    # - https://jupyterassistant.anacondaconnect.com (after migration: https://stage.anaconda.com/api/assistant)
    production: str = ConfigValueFactory().defaults["ANACONDA_ASSISTANT_API_URL"]  # type: ignore
    development: str = "https://stage.anaconda.com/api/assistant"  # type: ignore


class AssistantConfig(TypedDict):
    environmentType: AssistantEnvironmentType
    apiUrl: AssistantEnvironmentAPIUrl


class Datadog(TypedDict):
    clientToken: str
    service: str


class Heap(TypedDict):
    clientId: str


class FeatureFlag(TypedDict):
    clientId: str
    overrides: Dict[str, Union[str, bool]]


# TODO: handle development environments migration in a separate PR
class CloudDevelopmentHosts(str, Enum):
    tulz: str = "tulz.pa-dev.com"
    bp: str = "bp.pa-dev.com"
    stage: str = "nb.stage.anaconda.com"


class CloudProductionHosts(str, Enum):
    nb: str = ConfigValueFactory().defaults["ANACONDA_NOTEBOOKS_URL"]


class Config(TypedDict):
    platform: OSPlatform
    jupyterlab: JupyterLab
    extension: Extension
    environment: Environment
    environment_v2: Environment
    api: API
    org: Org
    static_content: StaticContent
    anaconda_auth: AnacondaAuth
    iam: IAM
    notebooks: Notebooks
    learning: Learning
    assistant: AssistantConfig
    datadog: Datadog
    heap: Heap
    featureFlag: FeatureFlag


def get_config() -> Config:
    cvfactory = ConfigValueFactory()

    try:
        overrides = json.loads(cvfactory["DEBUG_FEATURE_FLAG_OVERRIDES"])
    except json.JSONDecodeError:
        warnings.warn("DEBUG_FEATURE_FLAG_OVERRIDES is not valid JSON. Using the default value (or empty).")
        # log a warning about the invalid environment variable
        overrides = cvfactory.defaults.get("DEBUG_FEATURE_FLAG_OVERRIDES", {})

    return Config(
        platform=OSPlatform(**get_os_info()),
        jupyterlab=JupyterLab(server=jupyterlab_server.__version__, client=jupyterlab.__version__),
        extension=Extension(version=extension_version),
        environment=get_environment(cvfactory),
        environment_v2=get_environment_v2(cvfactory),
        api=API(
            domain=cvfactory["ANACONDA_API_DOMAIN"],
            url=cvfactory["ANACONDA_API_URL"],
        ),
        org=Org(url=cvfactory["ANACONDA_ORG_URL"], api=cvfactory["ANACONDA_ORG_API_URL"]),
        static_content=StaticContent(url=cvfactory["ANACONDA_STATIC_CONTENT_URL"]),
        anaconda_auth=AnacondaAuth(
            domain=cvfactory["ANACONDA_AUTH_DOMAIN"], token_endpoint=cvfactory["ANACONDA_AUTH_TOKEN_ENDPOINT"]
        ),
        iam=IAM(url=cvfactory["ANACONDA_IAM_URL"]),
        notebooks=Notebooks(url=cvfactory["ANACONDA_NOTEBOOKS_URL"]),
        learning=Learning(url=cvfactory["ANACONDA_LEARNING_URL"]),
        assistant=AssistantConfig(
            environmentType=cvfactory["ANACONDA_ASSISTANT_ENVIRONMENT_TYPE"],
            apiUrl=cvfactory["ANACONDA_ASSISTANT_API_URL"],
        ),
        datadog=Datadog(
            clientToken=cvfactory["DATADOG_CLIENT_TOKEN"],
            service=cvfactory["DATADOG_SERVICE"],
        ),
        heap=Heap(clientId=cvfactory["HEAP_CLIENT_ID"]),
        featureFlag=FeatureFlag(
            clientId=cvfactory["LAUNCH_DARKLY_CLIENT_ID"],
            overrides=overrides,
        ),
    )


def get_environment(cvfactory: ConfigValueFactory) -> Environment:
    if bool(cvfactory["LOCAL"]) or bool(cvfactory["LOCAL_DEVELOPMENT"]):
        return Environment.local

    if cvfactory["ANACONDA_API_URL"] == "https://anaconda.com/api":
        return Environment.production

    return Environment.development


def get_os_info():
    return {
        "name": platform.platform(),
        "system": platform.system(),
        "version": platform.version(),
        "release": platform.release(),
    }


def get_anaconda_org_platform() -> Optional[str]:
    """
    Map the get_os_info result to the platforms in anaconda.org
    Ex:
      {
          "osx-arm64": "4.1.0",
          "linux-64": "4.1.0",
          "linux-aarch64": "4.1.0",
          "osx-64": "4.1.0",
          "win-64": "4.1.0",
          "linux-ppc64le": "0.4.0"
      }
    """
    info = get_os_info()
    system = info.get("system", "").lower()
    machine = info.get("name", "").lower()

    platform_mapping = {
        "darwin": {
            "arm64": "osx-arm64",
            "aarch64": "osx-arm64",
            "x86_64": "osx-64",
            "i386": "osx-64",
            "i686": "osx-64",
        },
        "linux": {
            "x86_64": "linux-64",
            "amd64": "linux-64",
            "aarch64": "linux-aarch64",
            "arm64": "linux-aarch64",
            "ppc64le": "linux-ppc64le",
        },
        "windows": {"x86_64": "win-64", "amd64": "win-64", "windows": "win-64"},
    }
    if system and machine:
        try:
            options = platform_mapping[system]
        except KeyError:
            return
        for machine_arch, conda_arch in options.items():
            if machine_arch in machine:
                return conda_arch
        return "win-64"


def get_environment_v2(cvfactory: ConfigValueFactory) -> Environment:
    """
    Determine the environment based on various flags and configuration values.

    This function attempts to infer the environment from environment variables,
    configuration values, and predefined hostnames. The following environments
    are supported in the precedence order.:

    - `github_actions`: execution in GitHub Actions
    - `local_development`: local development environment (e.g., running with `LOCAL_DEVELOPMENT` set)
    - `local_production`: local production environment (e.g., running on a server with `LOCAL_PRODUCTION` or `LOCAL` set)
    - `cloud_production`: cloud production environment (e.g., running on PythonAnywhere with a valid site ID)
    - `cloud_development`: cloud development environment (e.g., running on a PythonAnywhere development host)

    If none of these environments can be determined, a ValueError is raised.

    Args:
        cvfactory: an instance of ConfigValueFactory to retrieve configuration values from

    Returns:
        the determined Environment instance
    """

    def str_to_bool(value):
        return str(value).strip().lower() in ["1", "true", "yes"]

    is_github_actions = str_to_bool(os.getenv("GITHUB_ACTIONS", "false"))
    is_local_development = str_to_bool(os.environ.get("LOCAL_DEVELOPMENT", "false"))
    is_local_production = str_to_bool(cvfactory["LOCAL"]) or str_to_bool(os.environ.get("LOCAL_PRODUCTION", "false"))

    if is_github_actions:
        return Environment.github_actions

    if is_local_development:
        return Environment.local_development

    # local production
    if not is_local_development and is_local_production:
        return Environment.local_production

    # cloud environments
    pythonanywhere_site = cvfactory["PYTHONANYWHERE_SITE"]
    if not is_local_development and bool(pythonanywhere_site):
        # cloud production
        for host in CloudProductionHosts:
            if pythonanywhere_site == host.value:
                return Environment.cloud_production

        # cloud development
        for host in CloudDevelopmentHosts:
            if pythonanywhere_site == host.value:
                return Environment.cloud_development

    warnings.warn("No valid environment flag is defined.")
    return Environment.cloud_production


SHARED_CONFIG = get_config()
