from typing import Dict

import httpx
from tornado.httputil import HTTPServerRequest

from aext_shared import logger as custom_logger
from aext_shared.backend_proxy import backend_proxy
from aext_shared.config import get_config
from aext_shared.utils import get_boolean_env_var

logger = custom_logger.logger
config = get_config()


class SnakeEyes:
    """Snake eyes client - Sends metrics/logs to Anaconda Snake Eyes"""

    async def send(self, incoming_request: HTTPServerRequest, metric_data: Dict) -> bool:
        """
        Send metric data to Snake Eyes
        Args:
            incoming_request: tornado incoming http request
            metric_data: json containing all the relevant data

        Returns: boolean indicating success/failure

        """

        if not get_boolean_env_var("COLLECT_METRICS", default=True):
            logger.debug("Metrics are OFF")
            return False

        try:
            logger.debug(f"Sending metric: {metric_data}")
            response = await backend_proxy(
                incoming_request,
                f"{config['api']['url']}/snake-eyes/record",
                method="POST",
                json=metric_data,
            )
            logger.debug(f"Metric response {response}")
            if 199 < response.status_code < 300:
                return True
            return False
        except httpx.TimeoutException:
            logger.warning("Timeout while writing file snake-eyes metrics")
            return False
        except httpx.NetworkError:
            logger.warning("Network error while sending snake-eyes metrics")
            return False
        except Exception:
            logger.warning("Error while sending snake-eyes metrics")
            return False
