import os
from unittest import mock

import anaconda_auth
import pytest

from aext_shared import Environment
from aext_shared.auth import require_api_key
from aext_shared.config import SHARED_CONFIG


class DummyHandler:
    def __init__(self):
        self.status_code = None
        self.message = None

    def set_status(self, status_code, msg):
        self.status_code = status_code
        self.message = msg

    @require_api_key
    async def get(self, *args, **kwargs):
        return True


@pytest.fixture
def mock_environment_is_local(is_local):
    with mock.patch.object(Environment, "is_local", return_value=is_local) as m:
        yield m


@pytest.mark.parametrize("is_local", [True])
async def test_require_api_key_should_return_403_if_token_not_set(mock_environment_is_local, is_local):
    handler = DummyHandler()
    # make sure user is logged out
    anaconda_auth.logout()
    await handler.get()

    assert handler.status_code == 403
    assert handler.message == "Forbidden: API Key is missing"


@pytest.mark.parametrize("is_local", [True])
async def test_require_api_key_with_logged_in_user(mock_environment_is_local, is_local):
    handler = DummyHandler()
    # make sure user is logged out
    anaconda_auth.logout()

    token_info = anaconda_auth.token.TokenInfo(api_key="dummy-key", domain="anaconda.com")
    token_info.save()
    get_return = await handler.get()

    assert handler.status_code is None
    assert handler.message is None
    assert get_return is True


@pytest.mark.parametrize("is_local", [False])
async def test_require_api_key_is_stored(mock_environment_is_local, is_local):
    handler = DummyHandler()
    # make sure user is logged out
    anaconda_auth.logout()
    get_return = await handler.get()

    assert handler.status_code is None
    assert handler.message is None
    assert get_return is True
