import os
import unittest
from copy import deepcopy
from unittest.mock import patch

import pytest

from aext_shared.config import Environment, get_anaconda_org_platform, get_config
from aext_shared.config_defaults_local import config_defaults_local

_default_config = get_config()  # no environment variables set


@patch.dict(os.environ, {"LOCAL_DEVELOPMENT": "1"}, clear=True)
def test_get_default_config_local_development():
    config = get_config()
    assert config["environment"] == Environment.local
    assert config["environment_v2"] == Environment.local_development


@patch.dict(
    os.environ,
    {
        "PYTHONANYWHERE_SITE": _default_config["notebooks"]["url"],
        "ANACONDA_NOTEBOOKS_URL": _default_config["notebooks"]["url"],
    },
    clear=True,
)
@patch.dict(os.environ, {"LOCAL": "true"}, clear=True)
def test_get_default_config_local_production():
    config = get_config()
    assert config["environment"] == Environment.local
    assert config["environment_v2"] == Environment.local_production


@patch.dict(
    os.environ,
    {"DEBUG_AEXT_SHARED_LOCAL": "true"},
    clear=True,
)
def test_get_default_config_debug():
    config = get_config()
    assert config["environment"] == Environment.local
    assert config["environment_v2"] == Environment.local_production
    assert config["datadog"] == {"clientToken": None, "service": None}
    assert config["heap"] == {"clientId": None}


@patch.dict(
    os.environ,
    {"DEBUG_AEXT_SHARED_LOCAL": "true", "DEBUG_FEATURE_FLAG_OVERRIDES": "None"},
    clear=True,
)
def test_get_config_with_invalid_feature_flag_overrides():
    config = get_config()
    assert config["environment"] == Environment.local
    assert config["environment_v2"] == Environment.local_production
    assert config["featureFlag"]["overrides"] == config_defaults_local.get("DEBUG_FEATURE_FLAG_OVERRIDES")


class TestGetEnvironmentV2(unittest.TestCase):

    @patch.dict(os.environ, {"GITHUB_ACTIONS": "1"}, clear=True)
    def test_github_actions(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.github_actions)

    @patch.dict(os.environ, {"LOCAL_DEVELOPMENT": "1"}, clear=True)
    def test_local_development(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.local_development)

    @patch.dict(os.environ, {"LOCAL_PRODUCTION": "1"}, clear=True)
    def test_local_production(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.local_production)

    @patch.dict(os.environ, {"PYTHONANYWHERE_SITE": _default_config["notebooks"]["url"]}, clear=True)
    def test_cloud_production(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_production)

    @patch.dict(os.environ, {"LOCAL": "False", "PYTHONANYWHERE_SITE": "tulz.pa-dev.com"}, clear=True)
    def test_cloud_development(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_development)

    @patch.dict(os.environ, {"LOCAL": "False", "PYTHONANYWHERE_SITE": "nb.stage.anaconda.com"}, clear=True)
    def test_cloud_development_stage(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_development)

    @patch.dict(os.environ, {}, clear=True)
    def test_default_env(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_production)

    @patch.dict(os.environ, {"PYTHONANYWHERE_SITE": "uknown.pa-dev.com"}, clear=True)
    def test_default_env_in_case_of_unknown_host(self):
        config = get_config()
        self.assertEqual(config["environment_v2"], Environment.cloud_production)


@pytest.mark.parametrize(
    "env, expected",
    [
        (Environment.local, True),
        (Environment.local_production, True),
        (Environment.local_development, True),
        (Environment.cloud_production, False),
        (Environment.cloud_development, False),
    ],
)
def test_environment_is_local(env, expected):
    assert Environment.is_local(env) == expected


@pytest.mark.parametrize(
    "env, expected",
    [
        (Environment.local, False),
        (Environment.local_production, False),
        (Environment.local_development, False),
        (Environment.cloud_production, True),
        (Environment.cloud_development, True),
    ],
)
def test_environment_is_cloud(env, expected):
    assert Environment.is_cloud(env) == expected


class TestGetAnacondaOrgPlatform(unittest.TestCase):

    @patch("aext_shared.config.get_os_info")
    def test_darwin_arm64(self, mock_get_os_info):
        mock_get_os_info.return_value = {"system": "Darwin", "name": "arm64"}
        self.assertEqual(get_anaconda_org_platform(), "osx-arm64")

    @patch("aext_shared.config.get_os_info")
    def test_darwin_x86_64(self, mock_get_os_info):
        mock_get_os_info.return_value = {"system": "Darwin", "name": "x86_64"}
        self.assertEqual(get_anaconda_org_platform(), "osx-64")

    @patch("aext_shared.config.get_os_info")
    def test_linux_x86_64(self, mock_get_os_info):
        mock_get_os_info.return_value = {"system": "Linux", "name": "x86_64"}
        self.assertEqual(get_anaconda_org_platform(), "linux-64")

    @patch("aext_shared.config.get_os_info")
    def test_linux_aarch64(self, mock_get_os_info):
        mock_get_os_info.return_value = {"system": "Linux", "name": "aarch64"}
        self.assertEqual(get_anaconda_org_platform(), "linux-aarch64")

    @patch("aext_shared.config.get_os_info")
    def test_linux_ppc64le(self, mock_get_os_info):
        mock_get_os_info.return_value = {"system": "Linux", "name": "ppc64le"}
        self.assertEqual(get_anaconda_org_platform(), "linux-ppc64le")

    @patch("aext_shared.config.get_os_info")
    def test_windows(self, mock_get_os_info):
        mock_get_os_info.return_value = {
            "system": "windows",
            "name": "x86_64",
        }
        self.assertEqual(get_anaconda_org_platform(), "win-64")

    @patch("aext_shared.config.get_os_info")
    def test_no_match(self, mock_get_os_info):
        # Simulate an OS that doesn't match any of the expected keys.
        mock_get_os_info.return_value = {"system": "UnknownOS", "name": "unknown"}
        self.assertIsNone(get_anaconda_org_platform())
