import logging
import os
import unittest
from unittest.mock import MagicMock, patch

from aext_shared.logger import CustomLogger


class TestCustomLogger(unittest.TestCase):
    def setUp(self):
        # Save original class variables to restore later
        self.original_jlab_logger = CustomLogger.jlab_logger
        self.original_general_logger = CustomLogger.general_logger

        # Create fresh mocks for each test
        self.mock_jlab_logger = MagicMock()
        self.mock_general_logger = MagicMock()

        # Replace class variables with our mocks
        CustomLogger.jlab_logger = self.mock_jlab_logger
        CustomLogger.general_logger = self.mock_general_logger

    def tearDown(self):
        # Restore original class variables
        CustomLogger.jlab_logger = self.original_jlab_logger
        CustomLogger.general_logger = self.original_general_logger

    @patch.dict(os.environ, {"LOG_LEVEL": str(logging.INFO)})
    def test_init_default_log_level(self):
        # Create an instance which should call setLevel during __init__
        custom_logger = CustomLogger()  # noqa

        # Verify setLevel was called with the correct level
        self.mock_general_logger.setLevel.assert_called_once_with(logging.INFO)

    @patch.dict(os.environ, {"LOG_LEVEL": str(logging.DEBUG)})
    def test_init_custom_log_level(self):
        # Create an instance with a custom log level
        custom_logger = CustomLogger()  # noqa

        # Verify setLevel was called with the correct custom level
        self.mock_general_logger.setLevel.assert_called_once_with(logging.DEBUG)

    def test_log_method(self):
        # Create an instance with our mocked loggers
        custom_logger = CustomLogger()

        # Test the _log method
        custom_logger._log("info", "Test message")

        # Verify both loggers' info methods were called
        self.mock_jlab_logger.info.assert_called_once_with("Test message")
        self.mock_general_logger.info.assert_called_once_with("Test message")

    def test_unset_loggers(self):
        # Create an instance
        custom_logger = CustomLogger()

        # Call the method being tested
        custom_logger._unset_loggers()

        # Verify the loggers were unset
        self.assertIsNone(custom_logger.jlab_logger)
        self.assertIsNone(custom_logger.general_logger)
