from unittest import mock
from unittest.mock import MagicMock

import httpx
import pytest
from tornado.httputil import HTTPServerRequest

from aext_shared.metrics import SnakeEyes


class DummyResponse:
    def __init__(self, status_code: int = 200):
        self.status_code = status_code


@pytest.fixture
def status_code():
    return 200


@pytest.fixture
def dummy_response(status_code: int):
    return DummyResponse(status_code=status_code)


@pytest.fixture
def mock_get_boolean_env_var(get_boolean_env_var_value: bool):
    with mock.patch("aext_shared.metrics.get_boolean_env_var") as m:
        m.return_value = get_boolean_env_var_value
        yield m


@pytest.fixture
def mock_backend_proxy(dummy_response):
    with mock.patch("aext_shared.metrics.backend_proxy") as m:
        m.return_value = dummy_response
        yield m


@pytest.mark.parametrize("get_boolean_env_var_value", [False])
async def test_send_metrics_off(get_boolean_env_var_value, mock_get_boolean_env_var, mock_backend_proxy):
    """
    If COLLECT_METRICS is False, send should immediately return False,
    and backend_proxy should not be called.
    """

    dummy_request = HTTPServerRequest("GET", "/dummy")
    client = SnakeEyes()
    result = await client.send(dummy_request, {"metric": "value"})

    assert result is False
    assert mock_backend_proxy.await_count == 0


@pytest.mark.parametrize("get_boolean_env_var_value, status_code", [(True, 200)])
async def test_send_success(get_boolean_env_var_value, mock_get_boolean_env_var, mock_backend_proxy):
    """
    When backend_proxy returns a 2xx response, send should return True.
    """

    dummy_request = HTTPServerRequest("GET", "/dummy")
    client = SnakeEyes()
    result = await client.send(dummy_request, {"metric": "value"})

    assert result is True
    assert mock_backend_proxy.await_count == 1


@pytest.mark.parametrize("get_boolean_env_var_value, status_code", [(True, 400)])
async def test_send_unsuccessful_status(get_boolean_env_var_value, mock_get_boolean_env_var, mock_backend_proxy):
    """
    When backend_proxy returns a non-2xx response, send should return False.
    """

    dummy_request = HTTPServerRequest("GET", "/dummy")
    client = SnakeEyes()
    result = await client.send(dummy_request, {"metric": "value"})

    assert result is False
    assert mock_backend_proxy.await_count == 1


async def test_send_timeout_exception(mock_backend_proxy):
    """
    If backend_proxy raises a httpx.TimeoutException, send should return False.
    """
    mock_backend_proxy.side_effect = httpx.TimeoutException("Timeout occurred")

    dummy_request = HTTPServerRequest("GET", "/dummy")
    client = SnakeEyes()
    result = await client.send(dummy_request, {"metric": "value"})

    assert result is False


async def test_send_network_error(mock_backend_proxy):
    """
    If backend_proxy raises a httpx.NetworkError, send should return False.
    """
    mock_backend_proxy.side_effect = httpx.NetworkError("Network Error")

    dummy_request = HTTPServerRequest("GET", "/dummy")
    client = SnakeEyes()
    result = await client.send(dummy_request, {"metric": "value"})

    assert result is False


async def test_send_generic_exception(mock_backend_proxy):
    """
    If backend_proxy raises a generic Exception, send should return False.
    """
    mock_backend_proxy.side_effect = Exception("Unknown Exception")

    dummy_request = HTTPServerRequest("GET", "/dummy")
    client = SnakeEyes()
    result = await client.send(dummy_request, {"metric": "value"})

    assert result is False
